<?php

/*

	Repository is an arbitrary list of directories that can be searched in order (cascade)

	Repository also collects matching urls as appropriate

	Repository also takes a list of filenames associated with the repository.

	Directories (locations) are in a list for cascading searches.

	Urls are in a list that is parallel (same keys) to the directory list, where matches are appropriate

	Searching directory lists (locations) with filenames yield lists of indexes of found directories (locations)

	Repositories can contain named Repositories

	Contained repositories, though arbitrary, are usually

		- useful partitions of the containing repositories, or

		- substructures (such as sub-directories) of the repositories

	Repositories can contain named "resources" which are diretory/url pairs



	API:

	// assumes that urls and locations are parallel, ie for every locations[$key] path there is a url[$key] (where there are parallel uri's)

	// child repositories

	set_repository($repostitoryname, $repository = NULL) // $repository name can be an associative array

	get_repository($repositoryname)

	// data content management

	set_location($location,$resourcename = NULL) // accepts arrays, add to list

	get_location($key)  // response to find...

	set_url($url,$resourcename = NULL,$key = NULL) // accepts arrays, add to list, must be same order as locations

	get_url($key)

	set_resource($location,$url = NULL,$resourcename = NULL) // $location and $url are guaranteed to have the same $key

	get_resource($resourcename)

	set_filename($filename) // add to list, useful for get_filespecs and get_fileurls

	find_first_location_index($filename) // where location contains filename, useful for finding subdirectories

	find_all_location_indexes($filename) // where locations contain filename

	get_filespecs($filename = NULL) // association of filespecs = filenames ([filename] = filespec) in locations; allows spec of filename(s)

	get_fileurls($filename = NULL) // association of filespecs = filenames ([filename] = filespec) in urls

*/

class Repository

{

	private $_locations = array();

	private $_urls = array();

	private $_filenames = array(); // somewhere in the repository

	private $_resources = array(); // named location/url pair

	// named child repositories

	private $_repositories = array();

	

	// accepts an associative array for $repositoryname

	public function set_repository($repositoryname, $repository = NULL)

	{

		if (is_array($repositoryname))

		{

			$this->_repositories = array_merge($this->_repositories,$repositoryname);

		}

		else

		{

			if ($repositoryname and $repository)

			{

				$this->_repositories[$repositoryname] = $repository;

			}

		}

	}

	public function get_repository($repositoryname)

	{

		if (array_key_exists($repositoryname,$this->_repositories))

		{

			return $this->_repositories[$repositoryname];

		}

		else

		{

			return NULL;

		}

	}

	

	public function set_location($location, $resourcename = NULL)

	{

		if (is_array($location))

		{

			$this->_locations = array_merge($this->_locations,$location);

			$index = NULL;

		}

		else

		{

			$this->_locations[] = $location;

			$index = (count($this->_locations) - 1);

			if (!is_null($resourcename))

			{

				$this->_resources[$resourcename]['location'] = $location;

			}

		}

		return $index;

	}

	

	public function get_location($key)

	{

		return $this->_locations[$key];

	}



	public function set_url($url, $resourcename=NULL, $index = NULL)

	{

		if (is_array($url))

		{

			$this->_urls = array_merge($this->_urls,$url);

		}

		else

		{

			if (is_null($index))

			{

				$this->_urls[] = $url;

			}

			else

			{

				$this->_urls[$index] = $url;

			}

			if (!is_null($resourcename))

			{

				$this->_resources[$resourcename]['url'] = $url;

			}

		}

	}



	public function get_url($key)

	{

		return $this->_urls[$key];

	}

	

	public function set_resource($location,$url = NULL,$resourcename = NULL)

	{

		$index = $this->set_location($location,$resourcename);

		if (!is_null($url))

		{

			$this->set_url($url,$resourcename,$index);

		}

	}

	

	public function get_resource($resourcename)

	{

		return array_key_exists($resourcename,$this->_resources)

			? $this->_resources[$resourcename]

			: NULL;

	}

	

	

	// $filename can be a list array

	public function set_filename($filename)

	{

		if (is_array($filename))

		{

			foreach ($filename as $value)

			{

				if (!in_array($value,$this->_filenames))

				{

					$this->_filenames[] = $value;

				}

			}

		}

		else

		{

			$this->_filenames[] = $filename;

		}

	}

	public function get_filenames()

	{

		return $this->_filenames;

	}

	// used in get_filespecs and get_fileurls

	public function find_first_location_index($filename)

	{

		foreach ($this->_locations as $key => $location)

		{

			if (file_exists($location . '/' . $filename))

			{

				return $key;

			}

		}

		return NULL;

	}

	// used directly for exceptions

	public function find_all_location_indexes($filename)

	{

		$fileindexes = array();

		foreach ($this->_locations as $key => $location)

		{

			if (file_exists($location . '/' . $filename))

			{

				$fileindexes[] = $key;

			}

		}

		return $fileindexes;

	}



	// get specs for first found filenames in locations

	public function get_filespecs($filename = NULL)

	{

		$filespecs = array();

		$filenames = is_null($filename)

			? $this->_filenames

			: (array) $filename;

			

		foreach ($filenames as $filename)

		{

			$key = $this->find_first_location_index($filename);

			if (!is_null($key))

			{

				$filespecs[$filename] = $this->_locations[$key] . '/' . $filename;

			}

		}

		return $filespecs;

	}

	// get urls for first found filenames in locations

	public function get_fileurls($filename = NULL)

	{

		$fileurls = array();

		$filenames = is_null($filename)

			? $this->_filenames

			: (array) $filename;

			

		foreach ($filenames as $filename)

		{

			$key = $this->find_first_location_index($filename);

			if (!is_null($key))

			{

				if (array_key_exists($key, $this->_urls))

				{

					$fileurls[$filename] = $this->_urls[$key] . '/' . $filename;

				}

			}

		}

		return $fileurls;

	}

	//accessor methods

	public function get_locations()

	{

		return $this->_locations;

	}

	public function get_urls()

	{

		return $this->_urls;

	}

}